'From Smalltalk-80 of June 28th, 1985 on 11 January 1987 at 11:54:32 am'!


Object subclass: #Fractal
	instanceVariableNames: 'matrix'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Graphics-Primitives'!

!Fractal methodsFor: 'generating'!
firstLevel
	matrix _ Array
		with: (Array with: 256 @ 20 @ 0)
		with: (Array with: 56 @ 280 @ 0 with: 456 @ 280 @ 0).!
nextLevel
	| newMatrix oldSize newSize l1 l2 l3 |
	oldSize _ matrix size.
	newSize _ oldSize * 2 - 1.
	l1 _ 200.4 / oldSize.
	l2 _ l1.
	l3 _ 241.7 / oldSize.

	newMatrix _ Array new: newSize.
	1 to: newSize do: [ :i |
		newMatrix at: i put: (Array new: i) ].

	oldSize to: 1 by: -1 do: [ :m |
		m to: 1 by: -1 do: [ :n |
			(newMatrix at: m * 2 - 1) at: n * 2 - 1 put: ((matrix at: m) at: n) ] ].

	3 to: newSize by: 2 do: [ :m |
		1 to: m - 2 by: 2 do: [ :n |
			(newMatrix at: m) at: n + 1 put:
				(((newMatrix at: m) at: n) middle: ((newMatrix at: m) at: n + 2)
					displaced: l3) ] ].

	1 to: newSize - 2 by: 2 do: [ :m |
		1 to: m by: 2 do: [ :n |
			(newMatrix at: m + 1) at: n put:
				(((newMatrix at: m) at: n) middle: ((newMatrix at: m + 2) at: n)
					displaced: l1).
			(newMatrix at: m + 1) at: n + 1  put:
				(((newMatrix at: m) at: n) middle: ((newMatrix at: m + 2) at: n + 2)
					displaced: l2) ] ].
			
	matrix _ newMatrix.
! !
!Fractal methodsFor: 'displaying'!
display
	| lineDrawer size |
	lineDrawer _ BitBlt destForm: Display
		sourceForm: (Form extent: 1@1) black
		halftoneForm: nil
		combinationRule: Form under
		destOrigin: 0@0
		sourceOrigin: 0@0
		extent: 1@1
		clipRect: Display boundingBox.
	size _ matrix size.

	1 to: size do: [ :m |
		1 to: m do: [ :n |
			(m < size) ifTrue:
				[ lineDrawer drawFrom: ((matrix at: m) at: n) asPoint
					to: ((matrix at: m + 1) at: n) asPoint.
				lineDrawer drawFrom: ((matrix at: m) at: n) asPoint
					to: ((matrix at: m + 1) at: n + 1) asPoint. ].
			(n < m) ifTrue:
				[ lineDrawer drawFrom: ((matrix at: m) at: n) asPoint
					to: ((matrix at: m) at: n + 1) asPoint. ] ] ].!
displayQD
	| size p q |
	size _ matrix size.

	1 to: size - 1 do: [ :m |
		1 to: m do: [ :n |
			q _ ((matrix at: m + 1) at: n) asPoint.
			Mac moveTo: q x v: q y.
			p _ ((matrix at: m) at: n) asPoint.
			Mac lineTo: p x v: p y.
			p _ ((matrix at: m + 1) at: n + 1) asPoint.
			Mac lineTo: p x v: p y.
			Mac lineTo: q x v: q y ] ].! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Fractal class
	instanceVariableNames: ''!

!Fractal class methodsFor: 'As yet unclassified'!
example
	| f |
	Cursor execute showWhile:
		[ f _ self new.
		f firstLevel.
		5 timesRepeat: [ f nextLevel ] ].
	Display white.
	f displayQD.
	
	"Fractal example"! !
	
!Random methodsFor: 'accessing'!
next
	"Answer with the next random number."

	^ seed _ (seed * 1.21723645262) fractionPart! !
!Random methodsFor: 'private'!
setSeed
	seed _ (Time millisecondClockValue bitAnd: 65535) / 65536.0
		"Time millisecondClockValue gives a large integer;  I only want the lower 16 bits."! !


!Point methodsFor: 'converting'!
@ zValue
	^ Point3D x: x y: y z: zValue! !

Point subclass: #Point3D
	instanceVariableNames: 'z '
	classVariableNames: 'RandomSource '
	poolDictionaries: ''
	category: 'Graphics-Primitives'!

!Point3D methodsFor: 'accessing'!
z
	^ z!
z: value
	^ z _ value! !
!Point3D methodsFor: 'converting'!
asPoint
	"Translate to 2D (integral) coordinates with some rotation.
	Answer integer coordinates cause bitblt likes it that way."
	
	| newX newY|
	newX _ x.
	newY _ y - (1.125 * z) truncated.
	^ newX @ newY!
middle: otherPoint displaced: range
	"Answer a point in the middle, randomly displaced.
	Keep coordinates integral for speed only."
	
	| newX newY newZ |
	newX _ (x + otherPoint x) // 2.
	newY _ (y + otherPoint y) // 2.
	newZ _ (z + otherPoint z) // 2.
	newZ _ newZ + (RandomSource next - 0.5 * range) truncated.
	^ Point3D x: newX y: newY z: newZ
! !
!Point3D methodsFor: 'printing'!
printOn: aStream 
	"The receiver prints on aStream in terms of infix notation."

	x printOn: aStream.
	aStream nextPut: $@.
	y printOn: aStream.
	aStream nextPut: $@.
	z printOn: aStream! !
!Point3D methodsFor: 'private'!
setX: xValue setY: yValue setZ: zValue

	x _ xValue.
	y _ yValue.
	z _ zValue! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Point3D class
	instanceVariableNames: ''!

!Point3D class methodsFor: 'class initialization'!
initialize
	RandomSource _ Random new

	"Point3D initialize"! !
!Point3D class methodsFor: 'instance creation'!
x: xValue y: yValue z: zValue
	"Answer an instance of me with coordinates xValue, yValue, and zValue."

	^self new setX: xValue setY: yValue setZ: zValue! !

Point3D initialize!


