!HeapObject methodsFor: 'As yet unclassified'!
highByteAt: offset
	"Answer the high byte at an offset in the receiver's data area.  The data is treated as unsigned integer."

	| y |
	y _ ((self valueAt: offset typeString: IasType) bitShift: -8) bitAnd: 16r000000FF.
	(y < 0)
		ifTrue: [ ^ y + 256 ]
		ifFalse: [ ^ y ]! !

!HeapArray class methodsFor: 'As yet unclassified'!
newWithPtr: n
	"Create a new instance of the reciever and allocate a data area to be accessed via it large enough for n elements."

	| newSelf |
	newSelf _ self basicNew.
	newSelf initialize.
	newSelf size: n.
	newSelf allocatePointer.
	^ newSelf! !
	
!HeapArray class methodsFor: 'As yet unclassified'!
newWithPtr
	"If a fixSize is given then answer an instance whoes data area is an array of that size, else error"

	(self fixSize == nil)
		ifTrue: [ ^ self error: 'No default size for HeapArray' ]
		ifFalse: [ ^ self newWithPtr: self fixSize ]! !
	
Transcript show: 'MusicConstants'; cr.!
Smalltalk at: #MusicConstants put: Dictionary new.!
MusicConstants at: #WaveForm put:  nil.!
MusicConstants at: #Tempo put:  80.!

Object subclass: #FourTone
	instanceVariableNames: 'ftSynth ftSound paramBlock '
	classVariableNames: 'SDrvrPitches '
	poolDictionaries: 'MusicConstants '
	category: 'Music'!

!FourTone methodsFor: 'demo stuff'!
setUpTest

	self duration: 60; rate: 16r50f97; phase: 0; wave: Wave square
	! !

!FourTone methodsFor: 'control'!
startSound
" ToolBox : StartSound (synthRec: Ptr; numBytes : LONGINT; completionRtn: ProcPtr);"

	Mac pbWrite: paramBlock async: false! !

!FourTone methodsFor: 'initialization'!
init
"initialize the receiver."

	ftSynth _ FTSynthRec newPointer.
	ftSynth mode: 1. "mode for four tone is always fTMode (=1)"
	ftSound _ FTSoundRec newPointer.
	ftSynth sndRec: (PascalRecord new pointer: ftSound pointer).
	ftSound sound1Rate: 0. 
	ftSound sound2Rate: 0.
	ftSound sound3Rate: 0.
	ftSound sound4Rate: 0.
	paramBlock _ HParamBlockRec newPointer.
	paramBlock ioRefNum: -4.
	paramBlock ioBuffer: (PascalRecord new pointer: ftSynth pointer).
	paramBlock ioReqCount: 256
! !

!FourTone methodsFor: 'accessing'!
duration: anInteger

	ftSound duration: anInteger!
phase: anInteger

	ftSound sound1Phase: anInteger!
rate: anInteger

	ftSound sound1Rate: anInteger!
sound: aChannel phase: aPhase

	ftSound sound: aChannel phase: aPhase!
sound: aChannel rate: aRate

	ftSound sound: aChannel rate: aRate!
sound: aChannel wave: aWave

	ftSound sound: aChannel wave: aWave!
wave: aWave

	ftSound sound1Wave: (PascalRecord new pointer: aWave pointer)! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

FourTone class
	instanceVariableNames: ''!

!FourTone class methodsFor: 'instance creation'!
new
"answer an instance of FourTone, used for four-voice synthesis."
	^ super new init
! !

!FourTone class methodsFor: 'constants'!
sdPitches
	"Pitches for Equal-tempered scale from IM SN-20. From three octaves below middle c to three above."

	^ #(16r604c 16r6606 16r6c16 16r7286 16r7953 16r808a 16r882f 16r9048 16r98dc 16ra7f3 16rab94 16rb5c8 16rc097 16rcc0c 16rd82d 16re50c 16rf2a6 16r10113 16r1105e 16r12090 16r13167 16r143e6 16r15728 16r16b91 16r1812e 16r19818 16r1b059 16r1ca18 16r1e54b 16r20227 16r220bd 16r24121 16r2636e 16r287cc 16r2ae50 16r2d722 16r3025d 16r33030 16r360b2 16r39430 16r3cd97 16r4044d 16r43e4f 16r48241 16r4c6dd 16r50f97 16r55ca1 16r5ae44 16r604b9 16r6605f 16r6c165 16r72861 16r7952e 16r8089b 16r882f3 16r9048b 16r98db9 16ra1f2f 16rab941 16rb5c87 16rc0972 16rcc0be 16rd82c9 16re50c2 16rf2a5b 16r101135 16r1105e6 16r120904 16r131b72 16r143e5d 16r157282 16r16b90f 16r1812e4 16r19817c 16r1b0593 16r1ca183 16r1e54b7 16r20226a 16r220bcc 16r241208 16r2636e4 16r287cbb 16r2ae505 16r2d721d)!
sDrvrPitches

	^ SDrvrPitches! !

!FourTone class methodsFor: 'class initialization'!
initialize	"FourTone initialize"

	SDrvrPitches _ self sdPitches! !

FourTone initialize!


HeapRecordPointers subclass: #FTSoundRec
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Music'!

!FTSoundRec methodsFor: 'access'!
duration
	<primitive: 161 recordOffset: 0 type: 'I'>
^self primitiveFailed!
duration: value
	<primitive: 161 recordOffset: 0 type: 'I'>
^self primitiveFailed!
sound1Phase
	<primitive: 161 recordOffset: 6 type: 'L'>
^self primitiveFailed!
sound1Phase: value
	<primitive: 161 recordOffset: 6 type: 'L'>
^self primitiveFailed!
sound1Rate
	<primitive: 161 recordOffset: 2 type: 'L'>
^self primitiveFailed!
sound1Rate: value
	<primitive: 161 recordOffset: 2 type: 'L'>
^self primitiveFailed!
sound1Wave
	<primitive: 161 recordOffset: 34 type: 'D'>
^self primitiveFailed!
sound1Wave: value
	<primitive: 161 recordOffset: 34 type: 'D'>
^self primitiveFailed!
sound2Phase
	<primitive: 161 recordOffset: 14 type: 'L'>
^self primitiveFailed!
sound2Phase: value
	<primitive: 161 recordOffset: 14 type: 'L'>
^self primitiveFailed!
sound2Rate
	<primitive: 161 recordOffset: 10 type: 'L'>
^self primitiveFailed!
sound2Rate: value
	<primitive: 161 recordOffset: 10 type: 'L'>
^self primitiveFailed!
sound2Wave
	<primitive: 161 recordOffset: 38 type: 'D'>
^self primitiveFailed!
sound2Wave: value
	<primitive: 161 recordOffset: 38 type: 'D'>
^self primitiveFailed!
sound3Phase
	<primitive: 161 recordOffset: 22 type: 'L'>
^self primitiveFailed!
sound3Phase: value
	<primitive: 161 recordOffset: 22 type: 'L'>
^self primitiveFailed!
sound3Rate
	<primitive: 161 recordOffset: 18 type: 'L'>
^self primitiveFailed!
sound3Rate: value
	<primitive: 161 recordOffset: 18 type: 'L'>
^self primitiveFailed!
sound3Wave
	<primitive: 161 recordOffset: 42 type: 'D'>
^self primitiveFailed!
sound3Wave: value
	<primitive: 161 recordOffset: 42 type: 'D'>
^self primitiveFailed!
sound4Phase
	<primitive: 161 recordOffset: 30 type: 'L'>
^self primitiveFailed!
sound4Phase: value
	<primitive: 161 recordOffset: 30 type: 'L'>
^self primitiveFailed!
sound4Rate
	<primitive: 161 recordOffset: 26 type: 'L'>
^self primitiveFailed!
sound4Rate: value
	<primitive: 161 recordOffset: 26 type: 'L'>
^self primitiveFailed!
sound4Wave
	<primitive: 161 recordOffset: 46 type: 'D'>
^self primitiveFailed!
sound4Wave: value
	<primitive: 161 recordOffset: 46 type: 'D'>
^self primitiveFailed!
sound: aChannel phase: aPhase

	(aChannel == 1) ifTrue: [^ self sound1Phase: aPhase].
	(aChannel == 2) ifTrue: [^ self sound2Phase: aPhase].
	(aChannel == 3) ifTrue: [^ self sound3Phase: aPhase].
	(aChannel == 4) ifTrue: [^ self sound4Phase: aPhase].!
sound: aChannel rate: aRate

	(aChannel == 1) ifTrue: [^ self sound1Rate: aRate].
	(aChannel == 2) ifTrue: [^ self sound2Rate: aRate].
	(aChannel == 3) ifTrue: [^ self sound3Rate: aRate].
	(aChannel == 4) ifTrue: [^ self sound4Rate: aRate].!
sound: aChannel wave: aWave

	(aChannel == 1) ifTrue: [^ self sound1Wave: aWave].
	(aChannel == 2) ifTrue: [^ self sound2Wave: aWave].
	(aChannel == 3) ifTrue: [^ self sound3Wave: aWave].
	(aChannel == 4) ifTrue: [^ self sound4Wave: aWave].! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

FTSoundRec class
	instanceVariableNames: ''!

!FTSoundRec class methodsFor: 'class constants'!
fieldString
	^ 'duration sound1Rate sound1Phase sound2Rate sound2Phase sound3Rate sound3Phase sound4Rate sound4Phase sound1Wave sound2Wave sound3Wave sound4Wave'!
length
	^ 50!
typeString
	^ 'I L L L L L L L L D D D D'! !

HeapRecordPointers subclass: #FTSynthRec
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Music'!

!FTSynthRec methodsFor: 'access'!
mode
	<primitive: 161 recordOffset: 0 type: 'I'>
^self primitiveFailed!
mode: value
	<primitive: 161 recordOffset: 0 type: 'I'>
^self primitiveFailed!
sndRec
	<primitive: 161 recordOffset: 2 type: 'D'>
^self primitiveFailed!
sndRec: value
	<primitive: 161 recordOffset: 2 type: 'D'>
^self primitiveFailed! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

FTSynthRec class
	instanceVariableNames: ''!

!FTSynthRec class methodsFor: 'class constants'!
fieldString
	^ 'mode sndRec'!
length
	^ 6!
typeString
	^ 'I D'! !

HeapArray subclass: #Wave
	instanceVariableNames: ''
	classVariableNames: 'SquareWave SineWave '
	poolDictionaries: 'MusicConstants '
	category: 'Music'!
Wave comment:
'This represents a 256-byte wavetable.'!

!Wave methodsFor: 'private'!
atOffset: off
	^ self byteAt: off!
atOffset: off put: value
	^ self byteAt: off put: value! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Wave class
	instanceVariableNames: ''!

!Wave class methodsFor: 'constants'!
elementLength
	^ 1!
elementType
	^ 'E'!
fixSize
	^ 256! !

!Wave class methodsFor: 'private'!
generateSin "create a sine wave of phase zero"
				"Wave generateSin"

	SineWave _ self newWithPtr.
	(0 to: 255) do:
		[:i | SineWave byteAt: i put: ((i / 127 * (Float pi)) sin + 1 * 127) asInteger].
	^ SineWave!
generateSquare
	| t1 |
	SquareWave _ self newWithPtr.
	(0 to: 127)
		do: [:t1 | SquareWave byteAt: t1 put: 255].
	(128 to: 255)
		do: [:t1 | SquareWave byteAt: t1 put: 0].
	^ SquareWave! !

!Wave class methodsFor: 'standard waves'!
sin "return a sine wave of phase zero"

	SineWave pointer == nil ifTrue: [^ Wave generateSin]
					ifFalse: [^ SineWave]!
square "return a square wave of phase zero"

	SquareWave pointer == nil ifTrue: [^ Wave generateSquare]
					ifFalse: [^ SquareWave]! !

!Wave class methodsFor: 'initialization'!
initialize

	"self generateSin."
	WaveForm _ self generateSquare!
startUp

	self initialize! !

Wave initialize!


OrderedCollection variableSubclass: #Chord
	instanceVariableNames: 'duration '
	classVariableNames: ''
	poolDictionaries: 'MusicConstants '
	category: 'Music'!
Chord comment:
'This class represents a collection of pitches with a common duration. (Basically a Note with multiple pitches)'!

!Chord methodsFor: 'accessing'!
addLast: newObject

	super addLast: newObject
	"Why oh why does OrderedCollection always return the added object rather than the OrderedCollection itself?"!
contents: aCollection

	self addAll: aCollection "Returning self, rather than aCollection"!
duration

	^ duration!
duration: aDuration

	duration _ aDuration! !

!Chord methodsFor: 'converting'!
asPitch
	"Return the pitch of the receiver's first stored note"

	^ self first! !

!Chord methodsFor: 'operating'!
chordInvert
	" invert the receiver by transposing the bottom pitch to the top."
	" N.B. this is a different operation than Note|invert or Pitch|invert, which invert a single note about a given pitch"

	| inv pitch |

	pitch _ self first.
	inv _ self copyFrom: 2 to: self size.
	(pitch absolutePitch < inv last absolutePitch)
		ifTrue: [inv add: (pitch octave: inv last octave + 1)]
		ifFalse: [inv add: (pitch octave: inv last octave)].
	inv duration: self duration.
	^ inv!
invert

	^ (Chord new contents: (self collect: [:p | p invert]))
		duration: duration!
invertAbout: aPitch

	^ (Chord new contents: (self collect: [:p | p invertAbout: aPitch]))
		duration: duration!
transposeBy: aPitch

	^ (Chord new contents: (self collect: [:p | p transposeBy: aPitch]))
		duration: duration! !

!Chord methodsFor: 'playing'!
play

	| fT |

	fT _ FourTone new.
	fT duration: (self duration asSDDuration).
	(1 to: self size) do: [:x | fT sound: x rate: ((self at: x) asSDPitch).
							fT sound: x wave: WaveForm.
							fT sound: x phase: 0].
	fT startSound.!
playWithFT: fT

	fT duration: (self duration asSDDuration).
	(1 to: self size) do: [:x | fT sound: x rate: ((self at: x) asSDPitch).
							fT sound: x wave: WaveForm.
							fT sound: x phase: 0].
	fT startSound.
	(1 to: self size) do: [:x | fT sound: x rate: 0] "clear voices (next event may have fewer voices)" ! !

!Chord methodsFor: 'printing'!
printOn: aStream

	super printOn: aStream.
	(duration ~~ Duration quarter) ifTrue: [duration printOn: aStream]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Chord class
	instanceVariableNames: ''!

!Chord class methodsFor: 'instance creation'!
cMajorTriad

	^ self majorTriad: (Pitch new: #C)!
contents: aCollection duration: dur
	"aCollection should be pitch names, like #C or 'FSharp(-2)'"

	| pitches |
	(aCollection isKindOf: Collection) ifFalse:
		[^ self error: 'chord contents must be a collection'].
	pitches _ OrderedCollection new.
	aCollection do: [:p | pitches addLast: (Pitch new: p)].
	^ (Chord new contents: pitches) 
		duration: Duration quarter!
contents: aCollection duration: dur octave: oct
	"aCollection should be pitch names, like #C or 'FSharp(-2)'"

	| pitches |
	(aCollection isKindOf: Collection) ifFalse:
		[^ self error: 'chord contents must be a collection'].
	pitches _ OrderedCollection new.
	aCollection do: [:p | pitches addLast: (Pitch new: p octave: oct)].
	^ (Chord new contents: pitches) 
		duration: Duration quarter!
majorTriad: aPitch

	^ (Chord with: (aPitch asPitch)
			with: (Pitch newValue: (aPitch pitchValue + 4))
			with: (Pitch newValue: (aPitch pitchValue + 7))) 
		duration: Duration quarter!
new

	^ super new duration: Duration quarter!
with: a
	"Assumes a is a Note, not a Pitch"

	^ (super with: a asPitch) duration: a duration!
with: a duration: dur
	"a can be Pitch or Note"

	(a isKindOf: Pitch) ifFalse: [^ self error: 'chord elements must be pitches or notes'].
	^ (super with: a asPitch) duration: dur!
with: a with: b

	^ (super with: a asPitch with: b asPitch) duration: a duration!
with: a with: b duration: dur

	(a isKindOf: Pitch) | (b isKindOf: Pitch) ifFalse: [^ self error: 'chord elements must be pitches or notes'].
	^ (super with: a asPitch with: b asPitch) duration: dur!
with: a with: b with: c

	^ (super with: a asPitch with: b asPitch with: c asPitch) duration: a duration!
with: a with: b with: c duration: dur

	(a isKindOf: Pitch) | (b isKindOf: Pitch) | (c isKindOf: Pitch) ifFalse: [^ self error: 'chord elements must be pitches or notes'].
	^ (super with: a asPitch with: b asPitch with: c asPitch) duration: dur! !

Fraction subclass: #Duration
	instanceVariableNames: ''
	classVariableNames: 'DurationNames '
	poolDictionaries: 'MusicConstants '
	category: 'Music'!
Duration comment:
'This class represents a musical time-dependence. It is represented as a fraction of a whole-note. The actual playing time is dependent upon the current tempo.'!

!Duration methodsFor: 'printing'!
printOn: aStream
	"will attempt to print the receiver's value in English. If it fails this (for durations of, say 5/8), it will default to Fraction|printOn:"

	| temp |
	temp _ self copy.
	(temp numerator == 3) "is it dotted?"
		ifTrue: [aStream nextPutAll: 'dotted '.
				temp setNumerator: 1 denominator: temp denominator / 2].
	((temp denominator rem: 3) == 0) "is it a triplet?"
		ifTrue: [aStream nextPutAll: 'triplet '.
				temp setNumerator: temp numerator denominator: temp denominator / 3 * 2].
	aStream nextPutAll: (DurationNames at: ((temp denominator log: 2) + 1.1) asInteger)
	"should recognize triplets, and default to Fraction|printOn: if too weird"! !

!Duration methodsFor: 'accessing'!
asNote 

	^ Note pitch: Pitch middleC duration: self! !

!Duration methodsFor: 'converting'!
asSDDuration

	^ self asSDDurationWithTempo: Tempo
!
asSDDurationWithTempo: QsPerMinute

	^ (self / QsPerMinute * 4 * 3600) asInteger "Measured in ticks (60ths of a second)"

	"QsPerMinute is the number of quarter notes per minute"! !

!Duration methodsFor: 'operating'!
dotted

	^ Duration numerator: (self numerator * 3) denominator: (self denominator * 2)!
triplet

	^ Duration numerator: numerator denominator: denominator / 2 * 3! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Duration class
	instanceVariableNames: ''!

!Duration class methodsFor: 'class initialization'!
initialize

	DurationNames _ #(whole half quarter eighth sixteenth thirtysecond)!
initializeDefaultTempo

	MusicConstants at: #Tempo put: 60! !

!Duration class methodsFor: 'instance creation'!
eighth

	^ self numerator: 1 denominator: 8!
fromString: aString
	"executes aString as messages to Duration. Example: Duration fromString: 'quarter dotted' will return a dotted quarter-note instance of Duration."

	| temp |
	temp _ self name , ' ' , aString asString.
	^ Compiler new evaluate: temp in: nil to: nil notifying: nil ifFail: []!
half

	^ self numerator: 1 denominator: 2!
quarter

	^ self numerator: 1 denominator: 4!
sixteenth

	^ self numerator: 1 denominator: 16!
thirtySecond

	^ self numerator: 1 denominator: 32!
whole

	^ self numerator: 1 denominator: 1! !

!Duration class methodsFor: 'tempo access'!
setTempo: aTempo

	Tempo _ aTempo! !

Duration initialize!


OrderedCollection variableSubclass: #Melody
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: 'MusicConstants '
	category: 'Music'!
Melody comment:
'This class represents a linear series of musical events. Each musical event may be a note, chord, or even another melody.'!

!Melody methodsFor: 'operating'!
invert

	^ self collect: [:n | n invert]!
invertAbout: aPitch 

	^ self collect: [:n | n invertAbout: aPitch]!
transposeBy: aPitch 

	^ self collect: [:n | n transposeBy: aPitch]! !

!Melody methodsFor: 'playing'!
play

	self playWithFT: FourTone new.
!
playWithFT: fT

	self do: [:note | note playWithFT: fT]! !

!Melody methodsFor: 'accessing'!
addFirst: newItem

	super addFirst: newItem "see Chord|addLast"!
addLast: newItem

	super addLast: newItem "see Chord|addLast"!
contents: aCollection

	self addAll: aCollection "Returning self, rather than aCollection"! !

!Melody methodsFor: 'converting'!
asPitch
	"just for completeness' sake..."
	^ self first asPitch! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Melody class
	instanceVariableNames: ''!

!Melody class methodsFor: 'instance creation'!
CMajor

	^  self contents: #(C D E F G A B 'C(1)')!
contents: aCollection 
	"aCollection should consist of note names, duration creation commands, 
	chord definitions, or octave specifications"

	| notes dur oct temp x |
	dur _ Duration quarter.
	oct _ 0. "no automatic octave transposition"
	"default duration for notes"
	notes _ OrderedCollection new: aCollection size.
	aCollection do: 
		[:p | 
		(p isKindOf: Number)
			ifTrue: [oct _ p] ifFalse:
		[(p isKindOf: String)
			ifTrue: 
				[temp _ p.
				(x _ temp findString: '(' startingAt: 1) > 0
					ifTrue: 
						[temp _ temp copyFrom: x + 1 to: temp size - 1.
						p _ p copyFrom: 1 to: x - 1].
				(Pitch pitchNames indexOf: p asSymbol)
					== 0
					ifTrue: [dur _ Duration fromString: p
						"change duration"]
					ifFalse: [notes addLast: (Note pitch: (Pitch new: p octave: oct)
								duration: dur copy)]]
			ifFalse: [notes addLast: (Chord contents: p duration: dur octave: oct) "assume its a collection -- meant for chords"]]].
	^ self new contents: notes! !

Magnitude subclass: #Pitch
	instanceVariableNames: 'pitchValue '
	classVariableNames: 'PitchNames '
	poolDictionaries: 'MusicConstants '
	category: 'Music'!

!Pitch methodsFor: 'arithmetic'!
* aPitch

	^ self copy setPitch: (pitchValue * (aPitch asPitch pitchValue))!
+ aPitch

	^ self copy setPitch: (pitchValue + (aPitch asPitch pitchValue))!
- aPitch

	^ self copy setPitch: (pitchValue - (aPitch asPitch pitchValue))!
/ aPitch

	^ self copy setPitch: (pitchValue / (aPitch asPitch pitchValue))!
negated

	^ self copy setPitch: pitchValue negated! !

!Pitch methodsFor: 'converting'!
absolutePitch
	"Return the pitchValue without octave adjustments (i.e. 1-12 only)"

	^ (pitchValue rem: 12) + 1. "twelve halftones in an octave"!
asNote

	^ Note pitch: self duration: Duration quarter
!
asPitch

	^ self!
asSDPitch

	pitchValue == 0 ifTrue: [^ 0].
	^ FourTone sDrvrPitches at: pitchValue! !

!Pitch methodsFor: 'accessing'!
octave

	^ (pitchValue quo: 12) - 3. "middleC is 48"
!
octave: anInteger
	"returns the receiver's absolute pitch in the indicated octave"
	^ self class newValue: (pitchValue == 0 ifTrue: [0]
								ifFalse: [anInteger + 4 * 12 + self absolutePitch])! !

!Pitch methodsFor: 'private'!
pitchValue

	^ pitchValue!
setPitch: anInteger

	pitchValue _ anInteger! !

!Pitch methodsFor: 'operations'!
invert

	^ self invertAbout: Pitch middleC!
invertAbout: aPitch

	^ aPitch asPitch * 2 - self !
transposeBy: aPitch

	^ self + aPitch asPitch - (Pitch new: #C)! !

!Pitch methodsFor: 'printing'!
printOn: aStream

	^ self printOn: aStream withKey: 0!
printOn: aStream withKey: aKey
	"aKey should be: 0 for no accidentals; < 0 flats (-1=1 flat, etc); > 0 sharps (ditto)"
	"to do: Put in code to check for double flats/sharps, naturals, etc (from key info)"

	| hTone octave |

	pitchValue == 0 ifTrue: [aStream nextPutAll: (PitchNames at: 13).
				^ self]. "null pitch (rest)"
	hTone _ self absolutePitch.
	octave _ self octave.
	"(PitchNames at: hTone) size > 1
		ifTrue: [aStream nextPutAll: ((PitchNames at: hTone) at: ( aKey < 0 ifTrue: [2] ifFalse: [1]))]
		ifFalse: ["

	aStream nextPutAll: (PitchNames at: hTone).
	(octave ~~ 0) ifTrue: [aStream nextPutAll: '(' , octave printString , ')'].! !

!Pitch methodsFor: 'comparing'!
< aPitch

	^ pitchValue < aPitch asPitch pitchValue!
= aPitch

	^ pitchValue = aPitch asPitch pitchValue!
hash

	^ pitchValue hash! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Pitch class
	instanceVariableNames: ''!

!Pitch class methodsFor: 'class initialization'!
initialize 		"Pitch initialize"

	"PitchNames _ #(C (CSharp DFlat) D (DSharp EFlat) E F (FSharp GFlat) G (GSharp AFlat) A (ASharp BFlat) B)"

	PitchNames _ #(C CSharp D DSharp E F FSharp G GSharp A ASharp B N).
	self initializeMusicConstants!
initializeMusicConstants	
	"Initialize constants shared by classes associated with music."

	(Smalltalk includes: MusicConstants) 
		ifFalse: [Smalltalk at: #MusicConstants put: (Dictionary new: 2)].
	self setDefaultWave: Wave square.
	Duration initializeDefaultTempo
	"Pitch initializeMusicConstants"! !

!Pitch class methodsFor: 'private'!
convertPitchName: aPitchName

	| pName x result |
	result _ 0.
	pName _ aPitchName asString.
	pName asSymbol == (PitchNames at: 13) ifTrue: [^ 0].
	(x _ pName findString: '(' startingAt: 1) > 0
		ifTrue: [result _ (pName copyFrom: x + 1 to: pName size - 1) asNumber.
				pName _ pName copyFrom: 1 to: x - 1].
	(x _ PitchNames indexOf: pName asSymbol) == 0
		ifTrue: [self error: 'bad pitch name'].
	result _ result * 12 + 35 + x.
	^ result
! !

!Pitch class methodsFor: 'instance creation'!
middleC

	^ self new: #C !
new: aPitchName

	^ self new: aPitchName octave: 0!
new: aPitchName octave: oct

	^ self newValue: (self convertPitchName: aPitchName) + (12 * oct)!
newValue: anInteger


	((anInteger < 0) and: [anInteger > 84])
		ifTrue: [self error: 'Pitch is out of range'].
	^ self new setPitch: anInteger! !

!Pitch class methodsFor: 'defaults'!
setDefaultWave: aWave

	MusicConstants at: #WaveForm put: aWave.
! !

!Pitch class methodsFor: 'constants'!
pitchNames

	^ PitchNames! !

Pitch initialize!


Pitch subclass: #Note
	instanceVariableNames: 'duration '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Music'!
Note comment:
'This class represents the fundamental real musical object. It is composed of a pitch and a duration.'!

!Note methodsFor: 'accessing'!
duration

	^ duration!
duration: aDuration

	duration _ aDuration!
pitch

	^ self asPitch! !

!Note methodsFor: 'converting'!
asNote

	^ self!
asPitch

	^ Pitch newValue: pitchValue! !

!Note methodsFor: 'private'!
pitch: aPitch duration: aDuration

	pitchValue _ aPitch pitchValue.
	duration _ aDuration! !

!Note methodsFor: 'printing'!
printOn: aStream

	duration printOn: aStream.
	aStream nextPutAll: '-note '.
	super printOn: aStream! !

!Note methodsFor: 'playing'!
play

	| fT |

	fT _ FourTone new.
	fT duration: (self duration asSDDuration).
	fT rate: self pitch asSDPitch.
	fT wave: WaveForm.
	fT phase: 0.
	fT startSound!
playWithFT: fT

	fT duration: (self duration asSDDuration).
	fT rate: self pitch asSDPitch.
	fT wave: WaveForm.
	fT phase: 0.
	fT startSound! !

!Note methodsFor: 'operating'!
dotted

	^ Note new pitch: self asPitch duration: duration dotted!
invert

	^ Note pitch: (super invert) duration: duration!
invertAbout: aPitch

	^ Note pitch: (super invertAbout: aPitch asPitch) duration: duration!
transposeBy: aPitch

	^ Note pitch: (super transposeBy: aPitch asPitch) duration: duration!
triplet

	^ Note new pitch: self asPitch duration: duration triplet! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Note class
	instanceVariableNames: ''!

!Note class methodsFor: 'instance creation'!
duration: aDuration

	^ Note pitch: Pitch middleC duration: aDuration!
new

	^ self pitch: Pitch middleC duration: Duration quarter!
pitch: aPitch

	^ Note pitch: aPitch duration: Duration quarter!
pitch: aPitch duration: aDuration

	^ super new pitch: aPitch duration: aDuration! !

